/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "valueindicator.h"

#include <limits>
#include <cmath>

#include <QDebug>

ValueIndicator::ValueIndicator() : QObject()
  , m_value(0)
{

}

qreal ValueIndicator::value() const
{
    return m_value;
}


ValueIndicatorDense::ValueIndicatorDense(int rows, int columns, const QList<ValueIndicator*>& data) : AbstractDense(rows, columns, data)
  , m_max(std::numeric_limits<double>::min())
  , m_min(std::numeric_limits<double>::max())
{

}

void ValueIndicatorDense::updateValues(const Dense& dense)
{
    m_max = std::numeric_limits<double>::min();
    m_min = std::numeric_limits<double>::max();
    for(int i = 0; i < dense.rows(); i++) {
        for(int j = 0; j < dense.columns(); j++) {
            double val = dense.value<double>(i, j);
            if (val > m_max) {
                m_max = val;
            }

            if (val < m_min) {
                m_min = val;
            }
        }
    }

    for(int i = 0; i < dense.rows(); i++) {
        for(int j = 0; j < dense.columns(); j++) {
            double val = dense.value<double>(i, j);

            value<ValueIndicator*>(i,j)->setValue((val - m_min)/(m_max - m_min));
        }
    }
}

ValueIndicatorDense::~ValueIndicatorDense()
{
    qDeleteAll(m_data);
}
