/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "visualizermodel.h"

#include <QDebug>

#include <qgrpcasyncreply.h>

#include "dense.h"

using namespace remotecontrol;
using namespace QtProtobuf;

VisualizerModel::VisualizerModel(std::shared_ptr<RemoteControlClient> &client, QObject *parent) : QObject(parent)
  , m_client(client)
{
    m_client->getConfiguration({}, this, [this](QGrpcAsyncReply *reply) {
        qDeleteAll(m_layers);
        m_networkConfig = reply->read<Configuration>();
        for(int i = 0; i < m_networkConfig.sizes().size(); i++) {
            m_layers.append(new NetworkLayerState);
            m_layers.last()->m_activations.setDimentions(m_networkConfig.sizes()[i], 1);
            QList<ValueIndicator*> data;
            for (int k = 0; k < m_networkConfig.sizes()[i]; k++) {
                data.append(new ValueIndicator);
            }
            m_layers.last()->m_activations.setData(data);

            if (i != 0) {
                int tolalItems = m_networkConfig.sizes()[i]*m_networkConfig.sizes()[i - 1];
                m_layers.last()->m_weights.setDimentions(m_networkConfig.sizes()[i], m_networkConfig.sizes()[i - 1]);
                data.clear();
                for (int k = 0; k < tolalItems; k++) {
                    data.append(new ValueIndicator);
                }
                m_layers.last()->m_weights.setData(data);
            }
        }
        sizesChanged();
    });

    QObject::connect(client.get(), &remotecontrol::RemoteControlClient::ActivationsUpdated, [this](const remotecontrol::LayerMatrix &activations) {
        if (m_layers.isEmpty()) {
            return;
        }
        Dense dense(activations.matrix().matrix());
        m_layers[activations.layer()]->m_activations.updateValues(Dense(activations.matrix().matrix()));
//        qDebug() << "ActivationsUpdated:" << dense.rows() << dense.columns() << activations.layer();
    });
    QObject::connect(client.get(), &remotecontrol::RemoteControlClient::BiasesUpdated, [this](const remotecontrol::LayerMatrix &biases) {
        if (m_layers.isEmpty()) {
            return;
        }
        Dense dense(biases.matrix().matrix());
//        qDebug() << "BiasesUpdated:" << dense.rows() << dense.columns();
    });
    QObject::connect(client.get(), &remotecontrol::RemoteControlClient::WeightsUpdated, [this](const remotecontrol::LayerMatrix &weights) {
        if (m_layers.isEmpty()) {
            return;
        }
         m_layers[weights.layer()]->m_weights.updateValues(Dense(weights.matrix().matrix()));
    });
    client->subscribeActivationsUpdates({});
    client->subscribeBiasesUpdates({});
    client->subscribeWeightsUpdates({});
}

ValueIndicator *VisualizerModel::activation(int layer, int row)
{
    ValueIndicator* indicator = m_layers[layer]->m_activations.value<ValueIndicator*>(row, 0);
    QQmlEngine::setObjectOwnership(indicator, QQmlEngine::CppOwnership);
    return indicator;
}

ValueIndicator *VisualizerModel::weight(int layer, int row, int column)
{
    ValueIndicator* indicator = m_layers[layer]->m_weights.value<ValueIndicator*>(row, column);
    QQmlEngine::setObjectOwnership(indicator, QQmlEngine::CppOwnership);
    return indicator;
}
